@@ -10,7 +10,7 @@ use tracing::debug;
 use url::Url;
 
 use uv_cache_key::CanonicalUrl;
-use uv_redacted::DisplaySafeUrl;
+use uv_redacted::{DisplaySafeUrl, DisplaySafeUrlError};
 use uv_small_str::SmallString;
 use uv_state::{StateBucket, StateStore};
 use uv_static::EnvVars;
@@ -284,7 +284,6 @@ impl PyxTokenStore {
 
     /// Read the tokens from the store.
     pub async fn read(&self) -> Result<Option<PyxTokens>, TokenStoreError> {
-        // Retrieve the API URL from the environment variable, or error if unset.
         if let Some(api_key) = read_pyx_api_key() {
             // Read the API key tokens from a file based on the API key.
             let digest = uv_cache_key::cache_digest(&api_key);
@@ -474,7 +473,7 @@ impl PyxTokenStore {
 #[derive(thiserror::Error, Debug)]
 pub enum TokenStoreError {
     #[error(transparent)]
-    Url(#[from] url::ParseError),
+    Url(#[from] DisplaySafeUrlError),
     #[error(transparent)]
     Io(#[from] io::Error),
     #[error(transparent)]
@@ -592,7 +591,7 @@ mod tests {
 
     #[test]
     fn test_is_known_url() {
-        let api_url = DisplaySafeUrl::from(Url::parse("https://api.pyx.dev").unwrap());
+        let api_url = DisplaySafeUrl::parse("https://api.pyx.dev").unwrap();
         let cdn_domain = "astralhosted.com";
 
         // Same realm as API.
@@ -647,7 +646,7 @@ mod tests {
 
     #[test]
     fn test_is_known_domain() {
-        let api_url = DisplaySafeUrl::from(Url::parse("https://api.pyx.dev").unwrap());
+        let api_url = DisplaySafeUrl::parse("https://api.pyx.dev").unwrap();
         let cdn_domain = "astralhosted.com";
 
         // Same realm as API.
