@@ -21,6 +21,9 @@ bitflags::bitflags! {
         const NATIVE_AUTH = 1 << 9;
         const S3_ENDPOINT = 1 << 10;
         const CACHE_SIZE = 1 << 11;
+        const INIT_PROJECT_FLAG = 1 << 12;
+        const WORKSPACE_METADATA = 1 << 13;
+        const WORKSPACE_DIR = 1 << 14;
     }
 }
 
@@ -42,6 +45,9 @@ impl PreviewFeatures {
             Self::NATIVE_AUTH => "native-auth",
             Self::S3_ENDPOINT => "s3-endpoint",
             Self::CACHE_SIZE => "cache-size",
+            Self::INIT_PROJECT_FLAG => "init-project-flag",
+            Self::WORKSPACE_METADATA => "workspace-metadata",
+            Self::WORKSPACE_DIR => "workspace-dir",
             _ => panic!("`flag_as_str` can only be used for exactly one feature flag"),
         }
     }
@@ -91,12 +97,14 @@ impl FromStr for PreviewFeatures {
                 "native-auth" => Self::NATIVE_AUTH,
                 "s3-endpoint" => Self::S3_ENDPOINT,
                 "cache-size" => Self::CACHE_SIZE,
+                "init-project-flag" => Self::INIT_PROJECT_FLAG,
+                "workspace-metadata" => Self::WORKSPACE_METADATA,
+                "workspace-dir" => Self::WORKSPACE_DIR,
                 _ => {
                     warn_user_once!("Unknown preview feature: `{part}`");
                     continue;
                 }
             };
-
             flags |= flag;
         }
 
