@@ -28,7 +28,7 @@ use uv_extract::hash::Hasher;
 use uv_fs::{Simplified, rename_with_retry};
 use uv_platform::{self as platform, Arch, Libc, Os, Platform};
 use uv_pypi_types::{HashAlgorithm, HashDigest};
-use uv_redacted::DisplaySafeUrl;
+use uv_redacted::{DisplaySafeUrl, DisplaySafeUrlError};
 use uv_static::EnvVars;
 
 use crate::PythonVariant;
@@ -56,7 +56,7 @@ pub enum Error {
     TooManyParts(String),
     #[error("Failed to download {0}")]
     NetworkError(DisplaySafeUrl, #[source] WrappedReqwestError),
-    #[error("Request failed after {retries} retries")]
+    #[error("Request failed after {retries} {subject}", subject = if *retries > 1 { "retries" } else { "retry" })]
     NetworkErrorWithRetries {
         #[source]
         err: Box<Error>,
@@ -75,9 +75,9 @@ pub enum Error {
         actual: String,
     },
     #[error("Invalid download URL")]
-    InvalidUrl(#[from] url::ParseError),
+    InvalidUrl(#[from] DisplaySafeUrlError),
     #[error("Invalid download URL: {0}")]
-    InvalidUrlFormat(Url),
+    InvalidUrlFormat(DisplaySafeUrl),
     #[error("Invalid path in file URL: `{0}`")]
     InvalidFileUrl(String),
     #[error("Failed to create download directory")]
@@ -109,7 +109,7 @@ pub enum Error {
     #[error("An offline Python installation was requested, but {file} (from {url}) is missing in {}", python_builds_dir.user_display())]
     OfflinePythonMissing {
         file: Box<PythonInstallationKey>,
-        url: Box<Url>,
+        url: Box<DisplaySafeUrl>,
         python_builds_dir: PathBuf,
     },
     #[error(transparent)]
@@ -411,6 +411,90 @@ impl PythonDownloadRequest {
         self.libc.as_ref()
     }
 
+    pub fn take_version(&mut self) -> Option<VersionRequest> {
+        self.version.take()
+    }
+
+    /// Remove default implementation and platform details so the request only contains
+    /// explicitly user-specified segments.
+    #[must_use]
+    pub fn unset_defaults(self) -> Self {
+        let request = self.unset_non_platform_defaults();
+
+        if let Ok(host) = Platform::from_env() {
+            request.unset_platform_defaults(&host)
+        } else {
+            request
+        }
+    }
+
+    fn unset_non_platform_defaults(mut self) -> Self {
+        self.implementation = self
+            .implementation
+            .filter(|implementation_name| *implementation_name != ImplementationName::default());
+
+        self.version = self
+            .version
+            .filter(|version| !matches!(version, VersionRequest::Any | VersionRequest::Default));
+
+        // Drop implicit architecture derived from environment so only user overrides remain.
+        self.arch = self
+            .arch
+            .filter(|arch| !matches!(arch, ArchRequest::Environment(_)));
+
+        self
+    }
+
+    #[cfg(test)]
+    pub(crate) fn unset_defaults_for_host(self, host: &Platform) -> Self {
+        self.unset_non_platform_defaults()
+            .unset_platform_defaults(host)
+    }
+
+    pub(crate) fn unset_platform_defaults(mut self, host: &Platform) -> Self {
+        self.os = self.os.filter(|os| *os != host.os);
+
+        self.libc = self.libc.filter(|libc| *libc != host.libc);
+
+        self.arch = self
+            .arch
+            .filter(|arch| !matches!(arch, ArchRequest::Explicit(explicit_arch) if *explicit_arch == host.arch));
+
+        self
+    }
+
+    /// Drop patch and prerelease information so the request can be re-used for upgrades.
+    #[must_use]
+    pub fn without_patch(mut self) -> Self {
+        self.version = self.version.take().map(VersionRequest::only_minor);
+        self.prereleases = None;
+        self.build = None;
+        self
+    }
+
+    /// Return a compact string representation suitable for user-facing display.
+    ///
+    /// The resulting string only includes explicitly-set pieces of the request and returns
+    /// [`None`] when no segments are explicitly set.
+    pub fn simplified_display(self) -> Option<String> {
+        let parts = [
+            self.implementation
+                .map(|implementation| implementation.to_string()),
+            self.version.map(|version| version.to_string()),
+            self.os.map(|os| os.to_string()),
+            self.arch.map(|arch| arch.to_string()),
+            self.libc.map(|libc| libc.to_string()),
+        ];
+
+        let joined = parts.into_iter().flatten().collect::<Vec<_>>().join("-");
+
+        if joined.is_empty() {
+            None
+        } else {
+            Some(joined)
+        }
+    }
+
     /// Iterate over all [`PythonDownload`]'s that match this request.
     pub fn iter_downloads<'a>(
         &'a self,
@@ -550,6 +634,30 @@ impl PythonDownloadRequest {
     }
 }
 
+impl TryFrom<&PythonInstallationKey> for PythonDownloadRequest {
+    type Error = LenientImplementationName;
+
+    fn try_from(key: &PythonInstallationKey) -> Result<Self, Self::Error> {
+        let implementation = match key.implementation().into_owned() {
+            LenientImplementationName::Known(name) => name,
+            unknown @ LenientImplementationName::Unknown(_) => return Err(unknown),
+        };
+
+        Ok(Self::new(
+            Some(VersionRequest::MajorMinor(
+                key.major(),
+                key.minor(),
+                *key.variant(),
+            )),
+            Some(implementation),
+            Some(ArchRequest::Explicit(*key.arch())),
+            Some(*key.os()),
+            Some(*key.libc()),
+            Some(key.prerelease().is_some()),
+        ))
+    }
+}
+
 impl From<&ManagedPythonInstallation> for PythonDownloadRequest {
     fn from(installation: &ManagedPythonInstallation) -> Self {
         let key = installation.key();
@@ -1131,7 +1239,7 @@ impl ManagedPythonDownload {
         let mut extracted = match uv_extract::strip_component(temp_dir.path()) {
             Ok(top_level) => top_level,
             Err(uv_extract::Error::NonSingularArchive(_)) => temp_dir.keep(),
-            Err(err) => return Err(Error::ExtractError(filename.to_string(), err)),
+            Err(err) => return Err(Error::ExtractError(filename, err)),
         };
 
         // If the distribution is a `full` archive, the Python installation is in the `install` directory.
@@ -1194,7 +1302,7 @@ impl ManagedPythonDownload {
     /// Download the managed Python archive into the cache directory.
     async fn download_archive(
         &self,
-        url: &Url,
+        url: &DisplaySafeUrl,
         client: &BaseClient,
         reporter: Option<&dyn Reporter>,
         python_builds_dir: &Path,
@@ -1262,12 +1370,12 @@ impl ManagedPythonDownload {
             let mut reader = ProgressReader::new(&mut hasher, progress_key, reporter);
             uv_extract::stream::archive(&mut reader, ext, target)
                 .await
-                .map_err(|err| Error::ExtractError(filename.to_string(), err))?;
+                .map_err(|err| Error::ExtractError(filename.to_owned(), err))?;
             reporter.on_request_complete(direction, progress_key);
         } else {
             uv_extract::stream::archive(&mut hasher, ext, target)
                 .await
-                .map_err(|err| Error::ExtractError(filename.to_string(), err))?;
+                .map_err(|err| Error::ExtractError(filename.to_owned(), err))?;
         }
         hasher.finish().await.map_err(Error::HashExhaustion)?;
 
@@ -1296,7 +1404,7 @@ impl ManagedPythonDownload {
         &self,
         python_install_mirror: Option<&str>,
         pypy_install_mirror: Option<&str>,
-    ) -> Result<Url, Error> {
+    ) -> Result<DisplaySafeUrl, Error> {
         match self.key.implementation {
             LenientImplementationName::Known(ImplementationName::CPython) => {
                 if let Some(mirror) = python_install_mirror {
@@ -1308,7 +1416,7 @@ impl ManagedPythonDownload {
                             self.url.to_string(),
                         ));
                     };
-                    return Ok(Url::parse(
+                    return Ok(DisplaySafeUrl::parse(
                         format!("{}/{}", mirror.trim_end_matches('/'), suffix).as_str(),
                     )?);
                 }
@@ -1323,7 +1431,7 @@ impl ManagedPythonDownload {
                             self.url.to_string(),
                         ));
                     };
-                    return Ok(Url::parse(
+                    return Ok(DisplaySafeUrl::parse(
                         format!("{}/{}", mirror.trim_end_matches('/'), suffix).as_str(),
                     )?);
                 }
@@ -1332,7 +1440,7 @@ impl ManagedPythonDownload {
             _ => {}
         }
 
-        Ok(Url::parse(&self.url)?)
+        Ok(DisplaySafeUrl::parse(&self.url)?)
     }
 }
 
@@ -1554,10 +1662,9 @@ where
 
 /// Convert a [`Url`] into an [`AsyncRead`] stream.
 async fn read_url(
-    url: &Url,
+    url: &DisplaySafeUrl,
     client: &BaseClient,
 ) -> Result<(impl AsyncRead + Unpin, Option<u64>), Error> {
-    let url = DisplaySafeUrl::from(url.clone());
     if url.scheme() == "file" {
         // Loads downloaded distribution from the given `file://` URL.
         let path = url
@@ -1570,7 +1677,7 @@ async fn read_url(
         Ok((Either::Left(reader), Some(size)))
     } else {
         let response = client
-            .for_host(&url)
+            .for_host(url)
             .get(Url::from(url.clone()))
             .send()
             .await
@@ -1584,7 +1691,7 @@ async fn read_url(
         // Check the status code.
         let response = response
             .error_for_status()
-            .map_err(|err| Error::from_reqwest(url, err, retry_count))?;
+            .map_err(|err| Error::from_reqwest(url.clone(), err, retry_count))?;
 
         let size = response.content_length();
         let stream = response
@@ -1598,6 +1705,7 @@ async fn read_url(
 
 #[cfg(test)]
 mod tests {
+    use crate::PythonVariant;
     use crate::implementation::LenientImplementationName;
     use crate::installation::PythonInstallationKey;
     use uv_platform::{Arch, Libc, Os, Platform};
@@ -1856,6 +1964,172 @@ mod tests {
         assert_eq!(downloads.len(), 0);
     }
 
+    #[test]
+    fn upgrade_request_native_defaults() {
+        let request = PythonDownloadRequest::default()
+            .with_implementation(ImplementationName::CPython)
+            .with_version(VersionRequest::MajorMinorPatch(
+                3,
+                13,
+                1,
+                PythonVariant::Default,
+            ))
+            .with_os(Os::from_str("linux").unwrap())
+            .with_arch(Arch::from_str("x86_64").unwrap())
+            .with_libc(Libc::from_str("gnu").unwrap())
+            .with_prereleases(false);
+
+        let host = Platform::new(
+            Os::from_str("linux").unwrap(),
+            Arch::from_str("x86_64").unwrap(),
+            Libc::from_str("gnu").unwrap(),
+        );
+
+        assert_eq!(
+            request
+                .clone()
+                .unset_defaults_for_host(&host)
+                .without_patch()
+                .simplified_display()
+                .as_deref(),
+            Some("3.13")
+        );
+    }
+
+    #[test]
+    fn upgrade_request_preserves_variant() {
+        let request = PythonDownloadRequest::default()
+            .with_implementation(ImplementationName::CPython)
+            .with_version(VersionRequest::MajorMinorPatch(
+                3,
+                13,
+                0,
+                PythonVariant::Freethreaded,
+            ))
+            .with_os(Os::from_str("linux").unwrap())
+            .with_arch(Arch::from_str("x86_64").unwrap())
+            .with_libc(Libc::from_str("gnu").unwrap())
+            .with_prereleases(false);
+
+        let host = Platform::new(
+            Os::from_str("linux").unwrap(),
+            Arch::from_str("x86_64").unwrap(),
+            Libc::from_str("gnu").unwrap(),
+        );
+
+        assert_eq!(
+            request
+                .clone()
+                .unset_defaults_for_host(&host)
+                .without_patch()
+                .simplified_display()
+                .as_deref(),
+            Some("3.13+freethreaded")
+        );
+    }
+
+    #[test]
+    fn upgrade_request_preserves_non_default_platform() {
+        let request = PythonDownloadRequest::default()
+            .with_implementation(ImplementationName::CPython)
+            .with_version(VersionRequest::MajorMinorPatch(
+                3,
+                12,
+                4,
+                PythonVariant::Default,
+            ))
+            .with_os(Os::from_str("linux").unwrap())
+            .with_arch(Arch::from_str("aarch64").unwrap())
+            .with_libc(Libc::from_str("gnu").unwrap())
+            .with_prereleases(false);
+
+        let host = Platform::new(
+            Os::from_str("linux").unwrap(),
+            Arch::from_str("x86_64").unwrap(),
+            Libc::from_str("gnu").unwrap(),
+        );
+
+        assert_eq!(
+            request
+                .clone()
+                .unset_defaults_for_host(&host)
+                .without_patch()
+                .simplified_display()
+                .as_deref(),
+            Some("3.12-aarch64")
+        );
+    }
+
+    #[test]
+    fn upgrade_request_preserves_custom_implementation() {
+        let request = PythonDownloadRequest::default()
+            .with_implementation(ImplementationName::PyPy)
+            .with_version(VersionRequest::MajorMinorPatch(
+                3,
+                10,
+                5,
+                PythonVariant::Default,
+            ))
+            .with_os(Os::from_str("linux").unwrap())
+            .with_arch(Arch::from_str("x86_64").unwrap())
+            .with_libc(Libc::from_str("gnu").unwrap())
+            .with_prereleases(false);
+
+        let host = Platform::new(
+            Os::from_str("linux").unwrap(),
+            Arch::from_str("x86_64").unwrap(),
+            Libc::from_str("gnu").unwrap(),
+        );
+
+        assert_eq!(
+            request
+                .clone()
+                .unset_defaults_for_host(&host)
+                .without_patch()
+                .simplified_display()
+                .as_deref(),
+            Some("pypy-3.10")
+        );
+    }
+
+    #[test]
+    fn simplified_display_returns_none_when_empty() {
+        let request = PythonDownloadRequest::default()
+            .fill_platform()
+            .expect("should populate defaults");
+
+        let host = Platform::from_env().expect("host platform");
+
+        assert_eq!(
+            request.unset_defaults_for_host(&host).simplified_display(),
+            None
+        );
+    }
+
+    #[test]
+    fn simplified_display_omits_environment_arch() {
+        let mut request = PythonDownloadRequest::default()
+            .with_version(VersionRequest::MajorMinor(3, 12, PythonVariant::Default))
+            .with_os(Os::from_str("linux").unwrap())
+            .with_libc(Libc::from_str("gnu").unwrap());
+
+        request.arch = Some(ArchRequest::Environment(Arch::from_str("x86_64").unwrap()));
+
+        let host = Platform::new(
+            Os::from_str("linux").unwrap(),
+            Arch::from_str("aarch64").unwrap(),
+            Libc::from_str("gnu").unwrap(),
+        );
+
+        assert_eq!(
+            request
+                .unset_defaults_for_host(&host)
+                .simplified_display()
+                .as_deref(),
+            Some("3.12")
+        );
+    }
+
     /// Test build display
     #[test]
     fn test_managed_python_download_build_display() {
