@@ -113,7 +113,7 @@ pub(crate) async fn pip_tree(
             client: &client,
             capabilities: &capabilities,
             prerelease,
-            exclude_newer,
+            exclude_newer: &exclude_newer,
             tags: Some(tags),
             requires_python: &requires_python,
         };
